module beamdata
  use const_and_precisions, only : wp_
  implicit none

  integer, parameter :: jmx=31,kmx=36,nmx=8000
  integer, save :: nray,nrayr,nrayth,nstep
  real(wp_) :: dst,rwmax
  real(wp_), dimension(:,:,:), allocatable, save :: psjki,ppabs,ccci,tauv,alphav
  real(wp_), dimension(:,:,:), allocatable, save :: pdjki,currj,didst
  integer, dimension(:,:), allocatable, save :: iiv,iop,iow,ihcd,istore
  real(wp_), dimension(:,:), allocatable, save :: tau1v
  real(wp_), dimension(:), allocatable, save :: q
  real(wp_), dimension(:,:,:), allocatable, save :: yyrfl !(6,:,:)
  real(wp_), dimension(:,:,:), allocatable, save :: ywrk,ypwrk !(6,:,:)
  real(wp_), dimension(:,:,:), allocatable, save :: xc,xco,du1,du1o !(3,:,:)
  real(wp_), dimension(:,:,:), allocatable, save :: gri,dgrad2v !(3,:,:)
  real(wp_), dimension(:,:,:,:), allocatable, save :: ggri !(3,3,:,:)
  real(wp_), dimension(:,:), allocatable, save :: grad2
  real(wp_), dimension(:), allocatable, save :: dffiu,ddffiu
  complex(wp_), dimension(:,:,:), allocatable, save :: ext,eyt

contains

  subroutine init_rtr(rtrparam)
    use gray_params, only : rtrparam_type
    implicit none
    type(rtrparam_type), intent(in) :: rtrparam

    dst=rtrparam%dst
    rwmax=rtrparam%rwmax
    nrayr=rtrparam%nrayr
    nrayth=rtrparam%nrayth
    if(nrayr==1) nrayth=1
    nray=(nrayr-1)*nrayth+1
    nstep=rtrparam%nstep
    call alloc_beam
!    call alloc_beam1
  end subroutine init_rtr

  function rayi2jk(i) result(jk)
    implicit none
    integer, intent(in) :: i
    integer, dimension(2) :: jk
    integer :: ioff

    if (i>1) then
      ioff  = i - 2
      jk(1) = ioff/nrayth                ! jr-2
      jk(2) = ioff - (jk(1))*nrayth + 1  ! kt
!     jk(2) = mod(ioff,nrayth) + 1       ! kt
      jk(1) = jk(1) + 2                  ! jr
    else
      jk = 1
    end if
  end function rayi2jk

  function rayi2j(i) result(jr)
    implicit none
    integer, intent(in) :: i
    integer :: jr

!   jr = max(1, (i-2)/nrayth + 2)
    if (i>1) then
      jr = (i-2)/nrayth + 2
    else
      jr = 1
    end if
  end function rayi2j

  function rayi2k(i) result(kt)
    implicit none
    integer, intent(in) :: i
    integer :: kt

!   kt = max(1, mod(i-2,nrayth) + 1)
    if (i>1) then
      kt = mod(i-2,nrayth) + 1
    else
      kt = 1
    end if
  end function rayi2k

  function rayjk2i(jr,kt) result(i)
    implicit none
    integer, intent(in) :: jr,kt
    integer :: i
    
!    i = max(1, (jr-2)*nrayth + kt + 1)
    if (jr>1) then
      i = (jr-2)*nrayth + kt + 1
    else
      i = 1
    end if
  end function rayjk2i
  
  subroutine alloc_beam
    implicit none
    
    call dealloc_beam
    allocate(psjki(nrayr,nrayth,nstep),  ppabs(nrayr,nrayth,nstep), &
             pdjki(nrayr,nrayth,nstep),   ccci(nrayr,nrayth,nstep), &
             currj(nrayr,nrayth,nstep),  didst(nrayr,nrayth,nstep), &
              tauv(nrayr,nrayth,nstep), alphav(nrayr,nrayth,nstep), &
                     iiv(nrayr,nrayth),          iop(nrayr,nrayth), &
                     iow(nrayr,nrayth),        tau1v(nrayr,nrayth), &
                    ihcd(nrayr,nrayth),       istore(nrayr,nrayth), &
                              q(nrayr),      yyrfl(6,nrayr,nrayth), &
                  ywrk(6,nrayr,nrayth),      ypwrk(6,nrayr,nrayth), &
                    xc(3,nrayr,nrayth),        xco(3,nrayr,nrayth), &
                   du1(3,nrayr,nrayth),       du1o(3,nrayr,nrayth), &
                   gri(3,nrayr,nrayth),    dgrad2v(3,nrayr,nrayth), &
                ggri(3,3,nrayr,nrayth),        grad2(nrayr,nrayth), &
                          dffiu(nrayr),              ddffiu(nrayr), &
                 ext(nrayr,nrayth,0:4),      eyt(nrayr,nrayth,0:4))
  end subroutine alloc_beam

!   subroutine alloc_beam1
!     implicit none
!     integer, intent(out) :: ierr
!     
!     call dealloc_beam
!     allocate(psjki(nray,nstep),  ppabs(nray,nstep), pdjki(nray,nstep),  &
!               ccci(nray,nstep),  currj(nray,nstep), didst(nray,nstep),  &
!               tauv(nray,nstep), alphav(nray,nstep),                     &
!                   ywrk(6,nray),      ypwrk(6,nray),   &
!                     xc(3,nray),        xco(3,nray),   &
!                    du1(3,nray),       du1o(3,nray),   &
!                    grad2(nray),    dgrad2v(3,nray),   &
!                    gri(3,nray),     ggri(3,3,nray),   &
!                   dffiu(nrayr),      ddffiu(nrayr),         q(nrayr),   &
!                      iiv(nray),          iop(nray),        iow(nray),   &
!                     ihcd(nray),       istore(nray),      tau1v(nray),   &
!                  yyrfl(6,nray),      ext(nray,0:4),    eyt(nray,0:4))
!   end subroutine alloc_beam1
! 
  subroutine dealloc_beam
    implicit none
    if (allocated(psjki))   deallocate(psjki)
    if (allocated(ppabs))   deallocate(ppabs)
    if (allocated(pdjki))   deallocate(pdjki)
    if (allocated(ccci))    deallocate(ccci)
    if (allocated(currj))   deallocate(currj)
    if (allocated(didst))   deallocate(didst)
    if (allocated(tauv))    deallocate(tauv)
    if (allocated(alphav))  deallocate(alphav)
    if (allocated(iiv))     deallocate(iiv)
    if (allocated(iop))     deallocate(iop)
    if (allocated(iow))     deallocate(iow)
    if (allocated(ihcd))    deallocate(ihcd)
    if (allocated(istore))  deallocate(istore)
    if (allocated(tau1v))   deallocate(tau1v)
    if (allocated(q))       deallocate(q)
    if (allocated(yyrfl))   deallocate(yyrfl)
    if (allocated(ywrk))    deallocate(ywrk)
    if (allocated(ypwrk))   deallocate(ypwrk)
    if (allocated(xc))      deallocate(xc)
    if (allocated(xco))     deallocate(xco)
    if (allocated(du1))     deallocate(du1)
    if (allocated(du1o))    deallocate(du1o)
    if (allocated(gri))     deallocate(gri)
    if (allocated(dgrad2v)) deallocate(dgrad2v)
    if (allocated(ggri))    deallocate(ggri)
    if (allocated(grad2))   deallocate(grad2)
    if (allocated(dffiu))   deallocate(dffiu)
    if (allocated(ddffiu))  deallocate(ddffiu)
    if (allocated(ext))     deallocate(ext)
    if (allocated(eyt))     deallocate(eyt)
  end subroutine dealloc_beam
end module beamdata
